@@ -226,8 +226,8 @@ make_lshd_options(void)
   self->with_publickey = 1;
   self->with_password = 1;
   self->with_tcpip_forward = 1;
-  /* Experimental, so disabled by default. */
-  self->with_x11_forward = 0;
+  /* Enabled by default. */
+  self->with_x11_forward = 1;
   self->with_pty = 1;
   self->subsystems = NULL;
 
@@ -352,15 +352,11 @@ main_options[] =
 
   { "login-auth-mode", OPT_LOGIN_AUTH_MODE, NULL, 0, 
     "Enable a telnet like mode (accept none-authentication and launch the" 
-    "login-program, making it responsible for authenticating the user).", 0 },
+    "login-shell, making it responsible for authenticating the user).", 0 },
 
   { "no-login-auth-mode", OPT_NO_LOGIN_AUTH_MODE, NULL, 0, 
     "Disable login-auth-mode (default).", 0 },
 
-  { "login-program", OPT_LOGIN_SHELL, "Program", 0,
-    "Use this as the login program for login-auth-mode, must " 
-    "authenticate users. ", 0 },
-
 #if 0
   { "login-auth-mode-user", OPT_LOGIN_AUTH_USER, "username", 0,
     "Run login-program as this user, defaults to the user "
@@ -410,9 +406,9 @@ main_options[] =
 #endif /* WITH_TCP_FORWARD */
 #if WITH_X11_FORWARD
   { "x11-forward", OPT_X11_FORWARD, NULL, 0,
-    "Enable x11 forwarding.", 0 },
+    "Enable x11 forwarding (default).", 0 },
   { "no-x11-forward", OPT_NO_X11_FORWARD, NULL, 0,
-    "Disable x11 forwarding (default).", 0 },
+    "Disable x11 forwarding.", 0 },
 #endif /* WITH_X11_FORWARD */
   
   { "subsystems", OPT_SUBSYSTEMS, "List of subsystem names and programs", 0,
@@ -561,10 +557,24 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  {
 	    int i = 0;
 	    self->kex_algorithms 
-	      = alloc_int_list(self->with_dh_keyexchange + self->with_srp_keyexchange);
+	      = alloc_int_list(3 * self->with_dh_keyexchange + self->with_srp_keyexchange);
 	    
 	    if (self->with_dh_keyexchange)
 	      {
+		struct keyexchange_algorithm *dh2
+		  = make_dh_server(make_dh14(self->random));
+		
+		LIST(self->kex_algorithms)[i++] = ATOM_DIFFIE_HELLMAN_GROUP14_SHA1;
+		ALIST_SET(self->super.algorithms,
+			  ATOM_DIFFIE_HELLMAN_GROUP14_SHA1,
+			  &dh2->super);
+
+		/* Alternative name for the same thing */
+		LIST(self->kex_algorithms)[i++] = ATOM_DIFFIE_HELLMAN_GROUP2_SHA1;
+		ALIST_SET(self->super.algorithms,
+			  ATOM_DIFFIE_HELLMAN_GROUP2_SHA1,
+			  &dh2->super);
+
 		LIST(self->kex_algorithms)[i++] = ATOM_DIFFIE_HELLMAN_GROUP1_SHA1;
 		ALIST_SET(self->super.algorithms,
 			  ATOM_DIFFIE_HELLMAN_GROUP1_SHA1,
@@ -649,8 +659,8 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	else if (self->with_loginauthmode)
 	  {
 	    const char *name;
-	    
-	    name = getenv(ENV_LOGNAME);
+
+	    USER_NAME_FROM_ENV(name);
 	    if (!name)
 	      argp_failure(state, EXIT_FAILURE, 0,
 			   "$LOGNAME not set in the environment.\n");
@@ -1039,14 +1049,12 @@ make_lshd_login_service(struct lshd_options *options)
      (name lshd_listen_callback)
      (params
        (handshake object handshake_info)
-       (kexinit object make_kexinit)
        (keys object alist)
        (logger object command)
        (services object command))
      (expr (lambda (lv)
-    	      (services (connection_handshake
+    	      (services (connection_handshake NULL
 	                   handshake
-			   kexinit
 			   keys
 			   (logger lv))))))
 */
@@ -1111,13 +1119,13 @@ make_lshd_listen_callback(struct lshd_options *options,
     CAST_SUBTYPE(command, server_callback,
 		 lshd_listen_callback
 		   (make_handshake_info(CONNECTION_SERVER,
-					"lsh - a free ssh",
+					SOFTWARE_SLOGAN,
 					NULL,
 					SSH_MAX_PACKET,
 					options->random,
 					options->super.algorithms,
+					kexinit,
 				        banner_text),
-		  kexinit,
 		  keys,
 		  logger,
 		  make_offer_service
